/*
---------------------------------------------------------------
SPDX-License-Identifier: BSD-3-Clause
 
Copyright (c) 2024, Renesas Electronics Corporation and/or its affiliates
 
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 
3. Neither the name of Renesas nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.
 
 
 
THIS SOFTWARE IS PROVIDED BY Renesas "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL RENESAS OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------
 
This project reads NDEF messages and controls the on-board LED D1.
If an NDEF Text record containing "ON" is found, the LED will turn on.
If the record contains "OFF", the LED will turn off.
Commands and LED pin number are easily customizable in code.
 
How to set up and run the example:
1. Prepare 2 cards with NDEF Text records containing the commands.
2. Open the Serial Monitor.
3. Approach the antenna with a card.
*/
 
#include <IotReader.h>
 
using namespace PtxIotReader;
 
const uint8_t LED = 7;
bool isON = false;
bool status;
 
// Configure polling for type A, B, F and V types
// with 500 ms idle time between cycles
// Low Power Card Detection enabled with regular polling at every 10th cycle
// and stand-by mode enabled
const PollingConfig pollConfig = {.pollTypeA = 1U,
                                  .pollTypeB = 1U,
                                  .pollTypeF212 = 1U,
                                  .pollTypeV = 1U,
                                  .idleTime = 500U,
                                  .discoverMode = 10U,
                                  .enableStandBy = 1U};
 
void setup() {
  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);
  Serial.begin(115200);
  while (!Serial);  // wait for the port to be opened
  // initialize the reader instance
  status = IotReader::getReader().begin();
 
  if (!status) {
    Serial.println("ERROR: failed to initialize IoT reader");
    return;
  }
 
  Serial.println("Place a card on the antenna to read it.");
  Serial.println();
}
 
void loop() {
  if (status) {
    std::vector<uint8_t> messageBuffer;
    if (IotReader::getReader().detectCard(pollConfig)) {
      // card ativated, attempt to read NDEF message
      if (IotReader::getReader().ndefRead(messageBuffer)) {
        if (messageBuffer.size() > 0) {
          const NdefRecord message =
              IotReader::getReader().getNdefMessage(messageBuffer);
          // check the NDEF message
          if (message.messageType == PtxIotReader::NdefMessageType::Text) {
            // get the record flags
            const auto flags = message.payload.front();
            // determine the language code length based on the flags
            const auto languageCodeLength = static_cast<uint8_t>(flags & 0x3F);
            // extract the command (usable text) from the message payload
            const auto command = std::string(message.payload.begin() + 1U + languageCodeLength, message.payload.end());
            Serial.print("Command: ");
            Serial.println(command.c_str());
            if (command == "ON" && !isON) {
              digitalWrite(LED, HIGH);
              isON = true;
              Serial.println("LED ON");
            }
            if (command == "OFF" && isON) {
              digitalWrite(LED, LOW);
              isON = false;
              Serial.println("LED OFF");
            }
          }
        }
      } else {
        Serial.println("No NDEF record found");
      }
      Serial.println();
    }
  }
}